%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%          Copyright (c) 2013 K. Neumann, CoR-Lab                     %%%
%%%          Bielefeld University, Germany, http://cor-lab.de           %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

classdef TSEDS < handle
    
    properties
        LyaFct;% = @(x) ( wsaqf.apply(x) );
        GradLyaFct;% = @(x) (wsaqf.gradient(x));
        VecField;
        dim=2;
        model = 'TSEDS';
    end
    
    methods
        function l = TSEDS(dim, spec)
            l.dim = dim;
            if nargin > 1
                for s=1:size(spec,1)
                    if ~strcmp(spec{s,1}, 'class')
                        try
                            l.(spec{s,1}) = spec{s,2};
                        catch me
                            disp(me.message);
                        end
                    end
                end
            end
        end
        
        function s = getDescriptorString(l)
            s = ['TSEDS']; 
        end
        
        function init(l, X)
            l.dim = size(X,2);
        end
        
        function Y = apply(l,X)
            Y = zeros(size(X,1),l.dim);
            for i = 1:size(X,1)
                if norm(X(1,:)) ~= 0
                    x = X(i,:)';
                    x_trafo = l.CoordTrafo(x')';      
                    gradient = l.GradLyaFct(x');
                    jacobian = (gradient*x')/(norm(x)*2*sqrt(l.LyaFct(x'))) + sqrt(l.LyaFct(x'))*(eye(l.dim)/norm(x) - (x*x')/(norm(x)^3));
                    Y(i,:) = (inv(jacobian)'*l.VecField(x_trafo))';
                else
                    Y(1,:) = zeros(dim,1);
                end
            end
        end
        
        function Y = apply_notrafo(l,X)
            Y = zeros(size(X,1),l.dim);
            for i = 1:size(X,1)
                if norm(X(1,:)) ~= 0
                    x = X(i,:)';
                    Y(i,:) = l.VecField(x);
                else
                    Y(1,:) = zeros(dim,1);
                end
            end
        end

        function [Xtrafo,Ytrafo] = CoordTrafo(l,X,Y)
            for i = 1:size(X,1)
                if norm(X(i,:)) ~= 0
                    Xtrafo(i,:) = (X(i,:)/norm(X(i,:)))*sqrt(l.LyaFct(X(i,:))); 
                else
                    Xtrafo(i,:) = zeros(1,l.dim);
                end
            end
            if nargin > 2
                for i = 1:size(Y,1)
                    if norm(Y(i,:)) ~= 0
                        x = X(i,:)';     
                        gradient = l.GradLyaFct(x');
                        jacobian = (gradient*x')/(norm(x)*2*sqrt(l.LyaFct(x'))) + sqrt(l.LyaFct(x'))*(eye(l.dim)/norm(x) - (x*x')/(norm(x)^3));
                        Ytrafo(i,:) = (jacobian'*Y(i,:)')';
                    else
                        Ytrafo(i,:) = zeros(1,l.dim);
                    end
                end
            end
        end
        
    end
end
