classdef WSAQF < Learner
   
    properties
        mu;
        P;
        P0;
        inp;
        tar;
        reg=0.0;
        w=0.5;
        num_L=1;
        MaxFunEvals=1000;
        model = 'WSAQF';
    end
    
    methods  

        function obj = WSAQF(inpDim, outDim, spec)
            obj = obj@Learner(inpDim, outDim, spec);
        end
        
        function s = getDescriptorString(obj)
            s = 'WSAQF'; 
        end
                
        function init(obj)
            theta0 = 5*randn(1,obj.num_L*obj.inpDim^2 + obj.inpDim^2 + obj.inpDim*obj.num_L);
            obj.Theta2Param(theta0);
        end

        function Val = apply(obj, X)
            X = X';
            Val = zeros(1,size(X,2));
            for i = 1:size(X,2)
                x = X(:,i);
                Val(i) = x'*obj.P0*x;
                for j = 1:obj.num_L
                    if x'*obj.P{j}*(x-obj.mu{j}) >= 0
                        beta = 1;
                    else
                        beta = 0;
                    end
                    Val(i) = Val(i) + beta*(x'*obj.P{j}*(x-obj.mu{j}))^2;
                end
            end   
        end

        function Val = gradient(obj, X)
            %Val = zeros(size(X,1),size(X,2));
            %for i = 1:size(X,2)
            %    x = X(:,i);
            %    Val(:,i) = obj.P0*x + obj.P0'*x;
            %    for j = 1:obj.num_L
            %        if x'*obj.P{j}*(x-obj.mu{j}) >= 0
            %            beta = 1;
            %        else
            %            beta = 0;
            %        end
            %        Val(:,i) = Val(:,i) + 2*beta*(x'*obj.P{j}*(x-obj.mu{j}))*(obj.P{j}*x+obj.P{j}'*x-obj.P{j}*obj.mu{j});
            %    end
            %end   
            X = X';
            inpDim = size(X,1);
            numX = size(X,2);
            Val = obj.P0*X + obj.P0'*X;
            for j = 1:obj.num_L
                beta =  diag((X'*obj.P{j})*(X-repmat(obj.mu{j},1,numX)))>=0;
                Val = Val + 2*repmat(beta',inpDim,1).*( repmat(diag(X'*obj.P{j}*(X-repmat(obj.mu{j},1,numX)))',inpDim,1) ).*(obj.P{j}*X+obj.P{j}'*X-obj.P{j}*repmat(obj.mu{j},1,numX));
            end
        end

        function Val = gradientTheta(obj, X, theta)
            X = X';
            obj.Theta2Param(theta);
            inpDim = size(X,1);
            numX = size(X,2);
            Val = obj.P0*X + obj.P0'*X;
            for j = 1:obj.num_L
                beta =  diag((X'*obj.P{j})*(X-repmat(obj.mu{j},1,numX)))>=0;
                Val = Val + 2*repmat(beta',inpDim,1).*( repmat(diag(X'*obj.P{j}*(X-repmat(obj.mu{j},1,numX)))',inpDim,1) ).*(obj.P{j}*X+obj.P{j}'*X-obj.P{j}*repmat(obj.mu{j},1,numX));
            end
        end
        
        function train(obj,X,Y)
            X = X'; Y = Y';
            obj.inp = X; obj.tar = Y;
            options = optimset('Algorithm','interior-point','MaxFunEvals',obj.MaxFunEvals);
            theta0 = obj.Param2Theta();
            theta = fmincon(@obj.crit,theta0,[],[],[],[],[],[],@obj.mycon,options);
            obj.Theta2Param(theta);
        end
        
        function Theta2Param(obj,theta)
            obj.P0 = reshape(theta(1:obj.inpDim^2),obj.inpDim,obj.inpDim);
            for i = 1:obj.num_L
                obj.P{i} = reshape(theta((1+obj.inpDim^2*i):(obj.inpDim^2+obj.inpDim^2*i)),obj.inpDim,obj.inpDim);
            end
            for i = 1:obj.num_L
                obj.mu{i} = theta(((obj.inpDim^2+obj.inpDim^2*obj.num_L)+1+obj.inpDim*(i-1)):((obj.inpDim^2+obj.inpDim^2*obj.num_L)+obj.inpDim+obj.inpDim*(i-1)))';
            end
        end
        
        function theta = Param2Theta(obj)
            theta = reshape(obj.P0,1,obj.inpDim^2);
            for i = 1:obj.num_L
                theta = [theta,reshape(obj.P{i},1,obj.inpDim^2)];
            end
            for i = 1:obj.num_L
                theta = [theta,reshape(obj.mu{i},1,obj.inpDim)];
            end
        end

%         function val = crit(obj,theta)
%             w = obj.w; inp = obj.inp; tar = obj.tar;
%             
%             val = 0;
%             for i = 1:size(inp,2);
%                 x = inp(:,i); v = tar(:,i);
%                 if x ~= 0
%                     gv = obj.GradVTheta(x,theta);
%                     psi = (gv'*v)/(norm(gv)*norm(v));
%                     val = val + (w*(1+sign(psi))/2)*psi^2 + ((1-w)/2)*psi^2;
%     %                 val = val + (((1+w)*sign(psi))/2)*psi^2 + ((1-w)/2)*psi^2;
%                 end                
%             end
%             val = val + obj.reg*norm(theta)^2;
%         end
        
        function val = crit(obj,theta)
            w = obj.w; inp = obj.inp; tar = obj.tar;
            
            gv = obj.gradientTheta(inp',theta);
            denotemp = sqrt(sum(gv.^2,1)).*sqrt(sum(tar.^2,1));%norm(gv)*norm(v)
            deno = denotemp;
            deno(denotemp==0)=1;
            psi = sum(gv.*tar)./deno;
            psi(denotemp==0)=0;
            val = (w*(1+sign(psi))/2).*psi.^2 + ((1-w)/2)*psi.^2;
%             val = ((1+w)/2)*sign(psi).*psi.^2 + ((1-w)/2)*psi.^2;
            val = sum(val);
            val = val + obj.reg*norm(theta)^2;
        end

        function [c,ceq] = mycon(obj,theta)
%             num_L = (size(theta,2)-4)/6;
%             P0 = reshape(theta(1:4),2,2);
%             for i = 1:num_L
%                 P{i} = reshape(theta((1+4*i):(4+4*i)),2,2);
%             end
            obj.Theta2Param(theta);
            c = -eig(.5*(obj.P0+obj.P0'));
            for i = 1:obj.num_L
                c = [c;-eig(.5*(obj.P{i}+obj.P{i}'))];
            end
            ceq = 0;
        end
        
    end
    
end