
%% XML
% <?xml version="1.0" encoding="UTF-8"?>
% <MLlib_XML_File>
%     <MachineLearningModel modelName="Mlp_Neural_Network">
%         <Configuration int_numInputNeurons="4" int_numLayers="1" bool_usesBias="true">
%             <MlpLayer1 int_numNeurons="6" str_activationFunction="ACT_FN_IDENTITY"/>
%         </Configuration>
%         <Parameters str_engine="mlp_fp32_engine" int_numLayers="1" bool_usesBias="true">
%             <MlpLayer1 str_type="dense" str_activationFunction="ACT_FN_IDENTITY">
%                 <WeightMatrix str_type="fp32" int_rows="6" int_columns="5">
%                     <Row1 arrfp32_vals="0.455443,0.301906,0.541781,-0.253339,-0.639187"/>
%                     <Row2 arrfp32_vals="0.891971,-0.129105,0.357915,-0.744195,-0.277263"/>
%                     <Row3 arrfp32_vals="-0.347602,0.323089,-0.912997,-0.23274,0.314637"/>
%                     <Row4 arrfp32_vals="-0.211747,-0.636116,-0.736092,0.0251508,0.696611"/>
%                     <Row5 arrfp32_vals="-0.099763,-0.42249,-0.127794,0.311451,-0.75072"/>
%                     <Row6 arrfp32_vals="0.692093,0.819697,-0.30946,0.30172,0.330007"/>
%                 </WeightMatrix>
%             </MlpLayer1>
%         </Parameters>
%     </MachineLearningModel>
% </MLlib_XML_File>

%% functions    
function xml = allmats2xml(Mats,Biases,EngNames,fnstr)

%     <Parameters str_engine="mlp_fp32_engine::mlp-eng01a" int_numLayers="3" bool_usesBias="true">
%     </Parameters>

    numParameterSets = length(Mats);

    xml = [];
    xml = [xml '<?xml version="1.0" encoding="UTF-8"?>\n'];
    xml = [xml '<MLlib_XML_File>\n'];
    xml = [xml '\t<MachineLearningModel modelName="Mlp_Neural_Network" defaultEngine="multi_engine_io_distributor::standard">\n'];
    xml = [xml func_Mat2XmlConf(Mats, Biases, '\t\t')];
    for i = 0:numParameterSets-1
        xml = [xml '\t\t<Parameters str_engine="mlp_fp32_engine::mlp-eng-' char(EngNames{i+1}) '" int_numLayers="1" bool_usesBias="true">\n'];
        xml = [xml func_Mat2XmlLayer(Mats{i+1}, Biases{i+1}, '\t\t\t')];
        xml = [xml '\t\t</Parameters>\n'];
    end
    
%     <IODistributor str_distributor="multi_engine_io_distributor::standard" str_engine_type="mlp_fp32_engine" int_engine_count="2">
%         <Engine0 str_engine_name="mlp-eng00" />
%         <Engine1 str_engine_name="mlp-eng01" />
%     </IODistributor>
    
    xml = [xml '\t\t<IODistributor str_distributor="multi_engine_io_distributor::standard" str_engine_type="mlp_fp32_engine" int_engine_count="' num2str(numParameterSets) '">\n'];
    for i = 0:numParameterSets-1
        xml = [xml '\t\t\t<Engine' num2str(i) ' str_engine_name="mlp-eng-' char(EngNames{i+1}) '"/>\n'];
    end
    xml = [xml '\t\t</IODistributor>\n'];

    xml = [xml '\t</MachineLearningModel>\n'];
    xml = [xml '</MLlib_XML_File>\n'];
    
    fileID = fopen(fnstr,'w');
    fprintf(fileID,xml);
    fclose(fileID);
end

function xml = func_Mat2XmlConf(Mat, Bias, tab)

    % <Configuration Configuration="4" int_numLayers="1" bool_usesBias="true">
    %     <MlpLayer1 int_numNeurons="6" str_activationFunction="ACT_FN_IDENTITY"/>
    % </Configuration>

    xml = [];
    xml = [xml tab '<Configuration int_numInputNeurons="' num2str(size(Mat,2)) '" int_numLayers="1" bool_usesBias="true">\n'];
    xml = [xml tab '\t' '<MlpLayer1 int_numNeurons="' num2str(size(Mat,1)) '" str_activationFunction="ACT_FN_IDENTITY"/>\n'];
    xml = [xml tab '</Configuration>\n'];
    
end

function xml = func_Mat2XmlLayer(Mat, Bias, tab)

    % <MlpLayer1 str_type="dense" str_activationFunction="ACT_FN_SIGMOID">
    %     <WeightMatrix str_type="fp32" int_rows="6" int_columns="5">
    %         <Row1 arrfp32_vals="0.455443,0.301906,0.541781,-0.253339,-0.639187"/>
    %         <Row2 arrfp32_vals="0.891971,-0.129105,0.357915,-0.744195,-0.277263"/>
    %         <Row3 arrfp32_vals="-0.347602,0.323089,-0.912997,-0.23274,0.314637"/>
    %         <Row4 arrfp32_vals="-0.211747,-0.636116,-0.736092,0.0251508,0.696611"/>
    %         <Row5 arrfp32_vals="-0.099763,-0.42249,-0.127794,0.311451,-0.75072"/>
    %         <Row6 arrfp32_vals="0.692093,0.819697,-0.30946,0.30172,0.330007"/>
    %     </WeightMatrix>
    % </MlpLayer1>

    [numRows,numCols] = size(Mat);
    
    xml = [];
    xml = [xml tab '<MlpLayer1 str_type="dense" str_activationFunction="ACT_FN_IDENTITY">\n'];
    xml = [xml tab '\t<WeightMatrix str_type="fp32" int_rows="' num2str(numRows) '" int_columns="' num2str(numCols+1) '">\n'];
    for i = 1:numRows
        xml = [xml tab '\t \t <Row' num2str(i) ' arrfp32_vals="' Row2Str(Mat(i,:)) ',' num2str(Bias(i)) '"/>\n'];
    end
    xml = [xml tab '\t</WeightMatrix>\n'];
    xml = [xml tab '</MlpLayer1>\n'];
    
end

function xml = Row2Str(row)
    xml = [];
    xml = [xml sprintf('%16.24e',row(1))];
    for i = 2:length(row)
        xml = [xml ',' sprintf('%16.24e',row(i))];
    end
end

% function xml = Row2Str(row)
%     xml = [];
%     xml = [xml num2str(row(1))];
%     for i = 2:length(row)
%         xml = [xml ',' num2str(row(i))];
%     end
% end