
function poly = fermiPoly(num_degree)
    %polynom of the fermi derivatives
    %a + b*f + c*f^2 + ... -> fliplr([a,b,c,...])
    
    if num_degree == 0
        poly = [1,0];
    elseif num_degree == 1
        poly = [-1,1,0];
    elseif num_degree == 2
        poly = [2,-3,1,0];
    elseif num_degree == 3
        poly = [-6,12,-7,1,0];
    elseif num_degree == 4
        poly = [24,-60,50,-15,1,0];
    elseif num_degree == 5    
        poly = [-120,360,-390,180,-31,1,0];
    elseif num_degree == 6    
        poly = [720,-2520,3360,-2100,602,-63,1,0];
    elseif num_degree == 7    
        poly = [-5040,20160,-31920,25200,-10206,1932,-127,1,0];
    elseif num_degree == 8    
        poly = [40320,-181440,332640,-317520,166824,-46620,6050,-255,1,0];
    else
        fermimat = fermiDerivative(num_degree);
        poly = [];
        [sX,sY] = size(fermimat);
        for i = 1:sX
            a = fermimat(i,1); m = fermimat(i,2); n = fermimat(i,3);
            val = zeros(1,m+n+1);
            c = 0;
            for k = m+1:n+m+1
                val(1,k) = (-1)^c*a*nchoosek(n,c);
                c = c + 1;
            end
            poly = [poly;val];
        end
        if sX > 1
            poly = sum(poly);
        end
        poly = fliplr(poly);
    end
    
end

